@@ -160,6 +160,7 @@ def macos_config(overrides):
         "compression_bgw",
         "compressed_collation",
     }
+    openssl_path = "/usr/local/opt/openssl@3"
     base_config = dict(
         {
             "cc": "clang",
@@ -168,11 +169,20 @@ def macos_config(overrides):
             "cxx": "clang++",
             "extra_packages": "",
             "ignored_tests": default_ignored_tests.union(macos_ignored_tests),
-            "os": "macos-13",
-            "pg_extra_args": "--enable-debug --with-libraries=/usr/local/opt/openssl@3/lib --with-includes=/usr/local/opt/openssl@3/include --without-icu",
+            "os": "macos-15-intel",
+            "pg_extra_args": (
+                " --enable-debug"
+                f" --with-libraries={openssl_path}/lib"
+                f" --with-includes={openssl_path}/include"
+                " --without-icu"
+            ),
             "pg_extensions": "postgres_fdw test_decoding",
             "pginstallcheck": True,
-            "tsdb_build_args": "-DASSERTIONS=ON -DREQUIRE_ALL_TESTS=ON -DOPENSSL_ROOT_DIR=/usr/local/opt/openssl@3",
+            "tsdb_build_args": (
+                " -DASSERTIONS=ON"
+                " -DREQUIRE_ALL_TESTS=ON"
+                f" -DOPENSSL_ROOT_DIR={openssl_path}"
+            ),
         }
     )
     base_config.update(overrides)
@@ -192,23 +202,37 @@ m["include"].append(build_debug_config({"pg": PG17_LATEST}))
 
 m["include"].append(build_debug_config({"pg": PG18_LATEST}))
 
-# Also test on ARM. See the available runners here:
+# Also test on ARM. The custom arm64 runner is only available in the
+# timescale/timescaledb repository.
+# See the available runners here:
 # https://github.com/timescale/timescaledb/actions/runners
-m["include"].append(
-    build_debug_config(
-        {
-            "pg": PG18_LATEST,
-            "os": "timescaledb-runner-arm64",
-            # We need to enable ARM crypto extensions to build the vectorized grouping
-            # code. The actual architecture for our ARM CI runner is reported as:
-            # -imultiarch aarch64-linux-gnu - -mlittle-endian -mabi=lp64 -march=armv8.2-a+crypto+fp16+rcpc+dotprod
-            "pg_extra_args": "--enable-debug --enable-cassert --without-llvm CFLAGS=-march=armv8.2-a+crypto",
-        }
+if os.environ.get("GITHUB_REPOSITORY") == "timescale/timescaledb":
+    m["include"].append(
+        build_debug_config(
+            {
+                "pg": PG18_LATEST,
+                "os": "timescaledb-runner-arm64",
+                # We need to enable ARM crypto extensions to build the vectorized grouping
+                # code. The actual architecture for our ARM CI runner is reported as:
+                # -imultiarch aarch64-linux-gnu - -mlittle-endian -mabi=lp64 -march=armv8.2-a+crypto+fp16+rcpc+dotprod
+                "pg_extra_args": "--enable-debug --enable-cassert --without-llvm CFLAGS=-march=armv8.2-a+crypto",
+            }
+        )
     )
-)
 
 # test timescaledb with release config on latest postgres release in MacOS
-m["include"].append(build_release_config(macos_config({"pg": PG18_LATEST})))
+# we only run compilation tests in pull requests.
+m["include"].append(
+    build_release_config(
+        macos_config(
+            {
+                "pg": PG18_LATEST,
+                "installcheck": not pull_request,
+                "pginstallcheck": not pull_request,
+            }
+        )
+    )
+)
 
 # Test latest postgres release without telemetry. Also run clang-tidy on it
 # because it's the fastest one.
@@ -334,7 +358,7 @@ elif len(sys.argv) > 2:
         stdout=subprocess.PIPE,
         shell=True,
     )
-    (output, err) = p.communicate()
+    output, err = p.communicate()
     p_status = p.wait()
     if p_status != 0:
         print(
