@@ -16,3 +16,6 @@ AS '@MODULE_PATHNAME@', 'ts_debug_point_id';
 
 CREATE OR REPLACE FUNCTION ts_now_mock()
 RETURNS TIMESTAMPTZ AS '@MODULE_PATHNAME@', 'ts_now_mock' LANGUAGE C STABLE STRICT PARALLEL SAFE;
+
+CREATE OR REPLACE FUNCTION clear_hypertable_cache() RETURNS VOID
+AS '@MODULE_PATHNAME@', 'ts_hypertable_cache_clear' LANGUAGE C;
