@@ -5,19 +5,17 @@
 SET LOCAL search_path TO pg_catalog, pg_temp;
 
 CREATE SCHEMA _timescaledb_catalog;
+CREATE SCHEMA _timescaledb_config;
 CREATE SCHEMA _timescaledb_functions;
 CREATE SCHEMA _timescaledb_internal;
 CREATE SCHEMA _timescaledb_cache;
-CREATE SCHEMA _timescaledb_config;
 CREATE SCHEMA timescaledb_experimental;
 CREATE SCHEMA timescaledb_information;
-CREATE SCHEMA _timescaledb_debug;
 
 GRANT USAGE ON SCHEMA
       _timescaledb_cache,
       _timescaledb_catalog,
       _timescaledb_config,
-      _timescaledb_debug,
       _timescaledb_functions,
       _timescaledb_internal,
       timescaledb_experimental,
