@@ -80,6 +80,8 @@ typedef struct CrossModuleFunctions
 	void (*set_rel_pathlist_dml)(PlannerInfo *, RelOptInfo *, Index, RangeTblEntry *, Hypertable *);
 	void (*set_rel_pathlist_query)(PlannerInfo *, RelOptInfo *, Index, RangeTblEntry *,
 								   Hypertable *);
+	void (*sort_transform_replace_pathkeys)(void *path, List *transformed_pathkeys,
+											List *original_pathkeys);
 
 	/* gapfill */
 	PGFunction gapfill_marker;
@@ -98,13 +100,9 @@ typedef struct CrossModuleFunctions
 	void (*tsl_postprocess_plan)(PlannedStmt *stmt);
 
 	/* Continuous Aggregates */
-	PGFunction partialize_agg;
-	PGFunction finalize_agg_sfunc;
-	PGFunction finalize_agg_ffunc;
 	DDLResult (*process_cagg_viewstmt)(Node *stmt, const char *query_string, void *pstmt,
 									   WithClauseResult *with_clause_options);
 	PGFunction continuous_agg_refresh;
-	PGFunction continuous_agg_process_hypertable_invalidations;
 	void (*continuous_agg_invalidate_raw_ht)(const Hypertable *raw_ht, int64 start, int64 end);
 	void (*continuous_agg_invalidate_mat_ht)(const Hypertable *raw_ht, const Hypertable *mat_ht,
 											 int64 start, int64 end);
@@ -116,9 +114,7 @@ typedef struct CrossModuleFunctions
 	PGFunction continuous_agg_validate_query;
 	PGFunction continuous_agg_get_bucket_function;
 	PGFunction continuous_agg_get_bucket_function_info;
-	PGFunction continuous_agg_migrate_to_time_bucket;
-	PGFunction cagg_try_repair;
-	PGFunction continuous_agg_read_invalidation_record;
+	PGFunction continuous_agg_get_grouping_columns;
 
 	PGFunction compressed_data_send;
 	PGFunction compressed_data_recv;
@@ -132,12 +128,15 @@ typedef struct CrossModuleFunctions
 	PGFunction create_compressed_chunk;
 	PGFunction compress_chunk;
 	PGFunction decompress_chunk;
+	PGFunction rebuild_columnstore;
 	void (*decompress_batches_for_insert)(ChunkInsertState *state, TupleTableSlot *slot);
 	void (*init_decompress_state_for_insert)(ChunkInsertState *state, TupleTableSlot *slot);
 	bool (*decompress_target_segments)(ModifyHypertableState *ht_state);
 
 	void (*columnstore_setup)(Hypertable *ht, WithClauseResult *with_clause_options);
-	RowCompressor *(*compressor_init)(Relation in_rel, BulkWriter **bulk_writer, bool sort);
+	RowCompressor *(*compressor_init)(Relation in_rel, BulkWriter **bulk_writer, bool sort,
+									  int tuple_sort_limit);
+	void (*compressor_set_invalidation)(RowCompressor *compressor, Hypertable *ht, Oid chunk_relid);
 	void (*compressor_add_slot)(RowCompressor *compressor, BulkWriter *bulk_writer,
 								TupleTableSlot *slot);
 	void (*compressor_flush)(RowCompressor *compressor, BulkWriter *bulk_writer);
@@ -151,6 +150,8 @@ typedef struct CrossModuleFunctions
 	 */
 	PGFunction compressed_data_decompress_forward;
 	PGFunction compressed_data_decompress_reverse;
+	PGFunction compressed_data_column_size;
+	PGFunction compressed_data_to_array;
 	PGFunction deltadelta_compressor_append;
 	PGFunction deltadelta_compressor_finish;
 	PGFunction gorilla_compressor_append;
@@ -164,6 +165,7 @@ typedef struct CrossModuleFunctions
 	PGFunction uuid_compressor_append;
 	PGFunction uuid_compressor_finish;
 	PGFunction bloom1_contains;
+	PGFunction bloom1_contains_any;
 	PGFunction (*bloom1_get_hash_function)(Oid type, FmgrInfo **finfo);
 
 	PGFunction create_chunk;
@@ -180,6 +182,8 @@ typedef struct CrossModuleFunctions
 
 	PGFunction detach_chunk;
 	PGFunction attach_chunk;
+
+	PGFunction estimate_compressed_batch_size;
 } CrossModuleFunctions;
 
 extern TSDLLEXPORT CrossModuleFunctions *ts_cm_functions;
