@@ -395,66 +395,6 @@ static FuncInfo funcinfo[] = {
 		.group_estimate = time_bucket_group_estimate,
 		.sort_transform = time_bucket_tz_sort_transform,
 	},
-	{
-		.origin = ORIGIN_TIMESCALE_EXPERIMENTAL,
-		.is_bucketing_func = true,
-		.allowed_in_cagg_definition = false,
-		.funcname = "time_bucket_ng",
-		.nargs = 2,
-		.arg_types = { INTERVALOID, DATEOID },
-		.group_estimate = time_bucket_group_estimate,
-		.sort_transform = time_bucket_sort_transform,
-	},
-	{
-		.origin = ORIGIN_TIMESCALE_EXPERIMENTAL,
-		.is_bucketing_func = true,
-		.allowed_in_cagg_definition = false,
-		.funcname = "time_bucket_ng",
-		.nargs = 3,
-		.arg_types = { INTERVALOID, DATEOID, DATEOID },
-		.group_estimate = time_bucket_group_estimate,
-		.sort_transform = time_bucket_sort_transform,
-	},
-	{
-		.origin = ORIGIN_TIMESCALE_EXPERIMENTAL,
-		.is_bucketing_func = true,
-		.allowed_in_cagg_definition = false,
-		.funcname = "time_bucket_ng",
-		.nargs = 2,
-		.arg_types = { INTERVALOID, TIMESTAMPOID },
-		.group_estimate = time_bucket_group_estimate,
-		.sort_transform = time_bucket_sort_transform,
-	},
-	{
-		.origin = ORIGIN_TIMESCALE_EXPERIMENTAL,
-		.is_bucketing_func = true,
-		.allowed_in_cagg_definition = false,
-		.funcname = "time_bucket_ng",
-		.nargs = 3,
-		.arg_types = { INTERVALOID, TIMESTAMPOID, TIMESTAMPOID },
-		.group_estimate = time_bucket_group_estimate,
-		.sort_transform = time_bucket_sort_transform,
-	},
-	{
-		.origin = ORIGIN_TIMESCALE_EXPERIMENTAL,
-		.is_bucketing_func = true,
-		.allowed_in_cagg_definition = false,
-		.funcname = "time_bucket_ng",
-		.nargs = 3,
-		.arg_types = { INTERVALOID, TIMESTAMPTZOID, TEXTOID },
-		.group_estimate = time_bucket_group_estimate,
-		.sort_transform = time_bucket_sort_transform,
-	},
-	{
-		.origin = ORIGIN_TIMESCALE_EXPERIMENTAL,
-		.is_bucketing_func = true,
-		.allowed_in_cagg_definition = false,
-		.funcname = "time_bucket_ng",
-		.nargs = 4,
-		.arg_types = { INTERVALOID, TIMESTAMPTZOID, TIMESTAMPTZOID, TEXTOID },
-		.group_estimate = time_bucket_group_estimate,
-		.sort_transform = time_bucket_sort_transform,
-	},
 	{
 		.origin = ORIGIN_TIMESCALE,
 		.is_bucketing_func = true,
@@ -526,6 +466,27 @@ static FuncInfo funcinfo[] = {
 		.sort_transform = time_bucket_gapfill_sort_transform,
 	},
 
+	{
+		.origin = ORIGIN_TIMESCALE,
+		.is_bucketing_func = false,
+		.allowed_in_cagg_definition = false,
+		.funcname = "first",
+		.nargs = 2,
+		.arg_types = { ANYELEMENTOID, ANYOID },
+		.group_estimate = NULL,
+		.sort_transform = NULL,
+	},
+	{
+		.origin = ORIGIN_TIMESCALE,
+		.is_bucketing_func = false,
+		.allowed_in_cagg_definition = false,
+		.funcname = "last",
+		.nargs = 2,
+		.arg_types = { ANYELEMENTOID, ANYOID },
+		.group_estimate = NULL,
+		.sort_transform = NULL,
+	},
+
 	{
 		.origin = ORIGIN_POSTGRES,
 		.is_bucketing_func = true,
@@ -550,6 +511,9 @@ static FuncInfo funcinfo[] = {
 
 #define _MAX_CACHE_FUNCTIONS (sizeof(funcinfo) / sizeof(funcinfo[0]))
 
+Oid ts_first_func_oid = InvalidOid;
+Oid ts_last_func_oid = InvalidOid;
+
 static HTAB *func_hash = NULL;
 
 static Oid
@@ -559,9 +523,10 @@ proc_get_oid(HeapTuple tuple)
 	return form->oid;
 }
 
-static void
-initialize_func_info()
+void
+ts_func_cache_init()
 {
+	Ensure(!func_hash, "function cache already initialized");
 	HASHCTL hashctl = {
 		.keysize = sizeof(Oid),
 		.entrysize = sizeof(FuncEntry),
@@ -615,6 +580,12 @@ initialize_func_info()
 
 		funcid = proc_get_oid(tuple);
 
+		/* Special handling for first/last to set up named variables for their oids */
+		if (strcmp(finfo->funcname, "first") == 0)
+			ts_first_func_oid = funcid;
+		else if (strcmp(finfo->funcname, "last") == 0)
+			ts_last_func_oid = funcid;
+
 		fentry = hash_search(func_hash, &funcid, HASH_ENTER, &hash_found);
 		Assert(!hash_found);
 		fentry->funcid = funcid;
@@ -630,8 +601,8 @@ ts_func_cache_get(Oid funcid)
 {
 	FuncEntry *entry;
 
-	if (NULL == func_hash)
-		initialize_func_info();
+	if (!func_hash)
+		ts_func_cache_init();
 
 	entry = hash_search(func_hash, &funcid, HASH_FIND, NULL);
 
