@@ -63,18 +63,14 @@ static const struct config_enum_entry compress_truncate_behaviour_options[] = {
 	{ NULL, 0, false }
 };
 
-#define GUC_CAGG_LOW_WORK_MEM_NAME "cagg_processing_low_work_mem"
-#define GUC_CAGG_LOW_WORK_MEM_VALUE (0.6 * 65536)
-#define GUC_CAGG_HIGH_WORK_MEM_NAME "cagg_processing_high_work_mem"
-#define GUC_CAGG_HIGH_WORK_MEM_VALUE (0.8 * 65536)
-
 bool ts_guc_enable_direct_compress_copy = false;
 bool ts_guc_enable_direct_compress_copy_sort_batches = true;
 bool ts_guc_enable_direct_compress_copy_client_sorted = false;
 int ts_guc_direct_compress_copy_tuple_sort_limit = 100000;
-bool ts_guc_enable_direct_compress_insert = false;
+TSDLLEXPORT bool ts_guc_enable_direct_compress_insert = false;
 bool ts_guc_enable_direct_compress_insert_sort_batches = true;
-bool ts_guc_enable_direct_compress_insert_client_sorted = false;
+TSDLLEXPORT bool ts_guc_enable_direct_compress_insert_client_sorted = false;
+TSDLLEXPORT bool ts_guc_enable_direct_compress_on_cagg_refresh = false;
 int ts_guc_direct_compress_insert_tuple_sort_limit = 10000;
 bool ts_guc_enable_deprecation_warnings = true;
 bool ts_guc_enable_optimizations = true;
@@ -86,6 +82,7 @@ bool ts_guc_enable_parallel_chunk_append = true;
 bool ts_guc_enable_runtime_exclusion = true;
 bool ts_guc_enable_constraint_exclusion = true;
 bool ts_guc_enable_qual_propagation = true;
+bool ts_guc_enable_qual_filtering = true;
 bool ts_guc_enable_cagg_reorder_groupby = true;
 TSDLLEXPORT bool ts_guc_enable_cagg_window_functions = false;
 bool ts_guc_enable_now_constify = true;
@@ -95,21 +92,15 @@ TSDLLEXPORT bool ts_guc_enable_cagg_sort_pushdown = true;
 #endif
 TSDLLEXPORT bool ts_guc_enable_cagg_watermark_constify = true;
 TSDLLEXPORT int ts_guc_cagg_max_individual_materializations = 10;
-TSDLLEXPORT bool ts_guc_enable_cagg_wal_based_invalidation = false;
-TSDLLEXPORT int ts_guc_cagg_wal_batch_size = 10000;
-TSDLLEXPORT int ts_guc_cagg_low_work_mem = GUC_CAGG_LOW_WORK_MEM_VALUE;
-TSDLLEXPORT int ts_guc_cagg_high_work_mem = GUC_CAGG_HIGH_WORK_MEM_VALUE;
 bool ts_guc_enable_osm_reads = true;
 TSDLLEXPORT bool ts_guc_enable_compressed_direct_batch_delete = true;
 TSDLLEXPORT bool ts_guc_enable_dml_decompression = true;
 TSDLLEXPORT bool ts_guc_enable_dml_decompression_tuple_filtering = true;
 TSDLLEXPORT int ts_guc_max_tuples_decompressed_per_dml = 100000;
-TSDLLEXPORT bool ts_guc_enable_transparent_decompression = true;
 TSDLLEXPORT bool ts_guc_enable_compression_wal_markers = false;
 TSDLLEXPORT bool ts_guc_enable_decompression_sorted_merge = true;
 bool ts_guc_enable_chunkwise_aggregation = true;
 bool ts_guc_enable_vectorized_aggregation = true;
-bool ts_guc_enable_custom_hashagg = false;
 TSDLLEXPORT bool ts_guc_enable_compression_indexscan = false;
 TSDLLEXPORT bool ts_guc_enable_bulk_decompression = true;
 TSDLLEXPORT bool ts_guc_auto_sparse_indexes = true;
@@ -127,6 +118,7 @@ TSDLLEXPORT int ts_guc_compression_batch_size_limit = 1000;
 TSDLLEXPORT bool ts_guc_compression_enable_compressor_batch_limit = false;
 TSDLLEXPORT CompressTruncateBehaviour ts_guc_compress_truncate_behaviour = COMPRESS_TRUNCATE_ONLY;
 bool ts_guc_enable_event_triggers = false;
+bool ts_guc_enable_chunk_auto_publication = false;
 bool ts_guc_debug_skip_scan_info = false;
 
 /* Only settable in debug mode for testing */
@@ -136,6 +128,7 @@ TSDLLEXPORT bool ts_guc_enable_compression_ratio_warnings = true;
 /* Enable of disable columnar scans for columnar-oriented storage engines. If
  * disabled, regular sequence scans will be used instead. */
 TSDLLEXPORT bool ts_guc_enable_columnarscan = true;
+TSDLLEXPORT bool ts_guc_enable_columnarindexscan = false;
 TSDLLEXPORT int ts_guc_bgw_log_level = WARNING;
 TSDLLEXPORT bool ts_guc_enable_skip_scan = true;
 #if PG16_GE
@@ -164,10 +157,6 @@ char *ts_telemetry_cloud = NULL;
 #endif
 
 TSDLLEXPORT char *ts_guc_license = TS_LICENSE_DEFAULT;
-char *ts_last_tune_time = NULL;
-char *ts_last_tune_version = NULL;
-
-bool ts_guc_debug_allow_cagg_with_deprecated_funcs = false;
 
 /*
  * Exit code for the scheduler.
@@ -338,38 +327,6 @@ get_segmentby_func(char *input_name)
 	return LookupFuncName(namelist, lengthof(argtyp), argtyp, true);
 }
 
-static bool
-check_cagg_low_work_mem(int *newval, void **extra, GucSource source)
-{
-	if (*newval >= ts_guc_cagg_high_work_mem)
-	{
-		GUC_check_errdetail("\"%s\" must be less than value of \"%s\".",
-							GUC_CAGG_LOW_WORK_MEM_NAME,
-							GUC_CAGG_HIGH_WORK_MEM_NAME);
-		GUC_check_errhint("Set \"%s\" to a value less than %d.",
-						  GUC_CAGG_LOW_WORK_MEM_NAME,
-						  ts_guc_cagg_high_work_mem);
-		return false;
-	}
-	return true;
-}
-
-static bool
-check_cagg_high_work_mem(int *newval, void **extra, GucSource source)
-{
-	if (*newval <= ts_guc_cagg_low_work_mem)
-	{
-		GUC_check_errdetail("\"%s\" must be greater than value of \"%s\".",
-							GUC_CAGG_HIGH_WORK_MEM_NAME,
-							GUC_CAGG_LOW_WORK_MEM_NAME);
-		GUC_check_errhint("Set \"%s\" to a value greater than %d.",
-						  GUC_CAGG_HIGH_WORK_MEM_NAME,
-						  ts_guc_cagg_low_work_mem);
-		return false;
-	}
-	return true;
-}
-
 static bool
 check_segmentby_func(char **newval, void **extra, GucSource source)
 {
@@ -435,6 +392,20 @@ ts_guc_default_orderby_fn_oid()
 	return get_orderby_func(ts_guc_default_orderby_fn);
 }
 
+/*
+ * Assign hook for chunk skipping.
+ *
+ * When chunk skipping is enabled, we need to clear the hypertable cache.
+ * Otherwise there might be cached entries without a valid range_space entry,
+ * which could lead to column stats not being created.
+ */
+static void
+chunk_skipping_assign_hook(bool newval, void *extra)
+{
+	if (newval)
+		ts_hypertable_cache_invalidate_callback();
+}
+
 void
 _guc_init(void)
 {
@@ -532,6 +503,18 @@ _guc_init(void)
 							 NULL,
 							 NULL);
 
+	DefineCustomBoolVariable(MAKE_EXTOPTION("enable_direct_compress_on_cagg_refresh"),
+							 "Enable direct compress on Continuous Aggregate refresh",
+							 "Enable experimental support for direct compression during Continuous "
+							 "Aggregate refresh",
+							 &ts_guc_enable_direct_compress_on_cagg_refresh,
+							 false,
+							 PGC_USERSET,
+							 0,
+							 NULL,
+							 NULL,
+							 NULL);
+
 	DefineCustomIntVariable(MAKE_EXTOPTION("direct_compress_insert_tuple_sort_limit"),
 							"Number of tuples that can be sorted at once in an INSERT operation",
 							"This is mainly used to keep the memory footprint down for "
@@ -649,6 +632,18 @@ _guc_init(void)
 							 NULL,
 							 NULL);
 
+	DefineCustomBoolVariable(MAKE_EXTOPTION("enable_qual_filtering"),
+							 "Enable qualifier filtering for chunks",
+							 "Filter qualifiers on chunks when complete chunk would be included by "
+							 "filter",
+							 &ts_guc_enable_qual_filtering,
+							 true,
+							 PGC_USERSET,
+							 0,
+							 NULL,
+							 NULL,
+							 NULL);
+
 	DefineCustomBoolVariable(MAKE_EXTOPTION("enable_qual_propagation"),
 							 "Enable qualifier propagation",
 							 "Enable propagation of qualifiers in JOINs",
@@ -711,17 +706,6 @@ _guc_init(void)
 							NULL,
 							NULL);
 
-	DefineCustomBoolVariable(MAKE_EXTOPTION("enable_transparent_decompression"),
-							 "Enable transparent decompression",
-							 "Enable transparent decompression when querying hypertable",
-							 &ts_guc_enable_transparent_decompression,
-							 true,
-							 PGC_USERSET,
-							 0,
-							 NULL,
-							 NULL,
-							 NULL);
-
 	DefineCustomBoolVariable(MAKE_EXTOPTION("enable_skipscan"),
 							 "Enable SkipScan",
 							 "Enable SkipScan for DISTINCT queries",
@@ -894,7 +878,7 @@ _guc_init(void)
 							 PGC_USERSET,
 							 0,
 							 NULL,
-							 NULL,
+							 chunk_skipping_assign_hook,
 							 NULL);
 
 	DefineCustomBoolVariable(MAKE_EXTOPTION("enable_segmentwise_recompression"),
@@ -992,6 +976,18 @@ _guc_init(void)
 							 NULL,
 							 NULL);
 
+	DefineCustomBoolVariable(MAKE_EXTOPTION("enable_chunk_auto_publication"),
+							 "Enable automatic chunk publication",
+							 "Enable automatically adding newly created chunks to the publication "
+							 "of their hypertable",
+							 &ts_guc_enable_chunk_auto_publication,
+							 false,
+							 PGC_USERSET,
+							 0,
+							 NULL,
+							 NULL,
+							 NULL);
+
 #ifdef TS_DEBUG
 	DefineCustomBoolVariable(MAKE_EXTOPTION("enable_null_compression"),
 							 "Debug only flag to enable NULL compression",
@@ -1016,63 +1012,6 @@ _guc_init(void)
 							 NULL,
 							 NULL);
 
-	DefineCustomBoolVariable(MAKE_EXTOPTION("enable_cagg_wal_based_invalidation"),
-							 "Enable experimental invalidations for continuous aggregates using "
-							 "WAL",
-							 "Use WAL to track changes to hypertables for continuous aggregates. "
-							 "This is not meant for production use.",
-							 &ts_guc_enable_cagg_wal_based_invalidation,
-							 false,
-							 PGC_USERSET,
-							 0,
-							 NULL,
-							 NULL,
-							 NULL);
-
-	DefineCustomIntVariable(MAKE_EXTOPTION("cagg_processing_wal_batch_size"),
-							"Batch size when processing WAL entries.",
-							"Number of entries processed from the WAL at a go. Larger values take "
-							"more memory but might be more efficient.",
-							&ts_guc_cagg_wal_batch_size,
-							10000,
-							1000,
-							10000000,
-							PGC_USERSET,
-							0,
-							NULL,
-							NULL,
-							NULL);
-
-	DefineCustomIntVariable(MAKE_EXTOPTION(GUC_CAGG_LOW_WORK_MEM_NAME),
-							"Low working memory limit for continuous aggregate invalidation "
-							"processing.",
-							"The low working memory limit for the continuous aggregate "
-							"invalidation processing.",
-							&ts_guc_cagg_low_work_mem,
-							GUC_CAGG_LOW_WORK_MEM_VALUE,
-							64,
-							MAX_KILOBYTES,
-							PGC_USERSET,
-							GUC_UNIT_KB,
-							check_cagg_low_work_mem,
-							NULL,
-							NULL);
-
-	DefineCustomIntVariable(MAKE_EXTOPTION(GUC_CAGG_HIGH_WORK_MEM_NAME),
-							"High working memory limit for continuous aggregate invalidation "
-							"processing.",
-							"The high working memory limit for the continuous aggregate "
-							"invalidation processing.",
-							&ts_guc_cagg_high_work_mem,
-							GUC_CAGG_HIGH_WORK_MEM_VALUE,
-							64,
-							MAX_KILOBYTES,
-							PGC_USERSET,
-							GUC_UNIT_KB,
-							check_cagg_high_work_mem,
-							NULL,
-							NULL);
-
 	DefineCustomBoolVariable(MAKE_EXTOPTION("enable_tiered_reads"),
 							 "Enable tiered data reads",
 							 "Enable reading of tiered data by including a foreign table "
@@ -1097,17 +1036,6 @@ _guc_init(void)
 							 NULL,
 							 NULL);
 
-	DefineCustomBoolVariable(MAKE_EXTOPTION("enable_custom_hashagg"),
-							 "Enable custom hash aggregation",
-							 "Enable creating custom hash aggregation plans",
-							 &ts_guc_enable_custom_hashagg,
-							 false,
-							 PGC_USERSET,
-							 0,
-							 NULL,
-							 NULL,
-							 NULL);
-
 	DefineCustomBoolVariable(MAKE_EXTOPTION("enable_vectorized_aggregation"),
 							 "Enable vectorized aggregation",
 							 "Enable vectorized aggregation for compressed data",
@@ -1181,12 +1109,10 @@ _guc_init(void)
 							 NULL);
 
 	DefineCustomBoolVariable(MAKE_EXTOPTION("enable_columnarscan"),
-							 "Enable columnar-optimized scans for supported access methods",
-							 "A columnar scan replaces sequence scans for columnar-oriented "
-							 "storage "
-							 "and enables storage-specific optimizations like vectorized filters. "
-							 "Disabling columnar scan will make PostgreSQL fall back to regular "
-							 "sequence scans.",
+							 "Enable ColumnarScan for columnar storage",
+							 "Transparently decompress columnar data using ColumnarScan custom "
+							 "node. Disabling columnar scan will ignore data stored in columnar "
+							 "format in queries.",
 							 &ts_guc_enable_columnarscan,
 							 true,
 							 PGC_USERSET,
@@ -1195,6 +1121,18 @@ _guc_init(void)
 							 NULL,
 							 NULL);
 
+	DefineCustomBoolVariable(MAKE_EXTOPTION("enable_columnarindexscan"),
+							 "Enable metadata-only optimization for ColumnarScans",
+							 "Enable experimental support for returning results directly from "
+							 "compression metadata without decompression",
+							 &ts_guc_enable_columnarindexscan,
+							 false,
+							 PGC_USERSET,
+							 0,
+							 NULL,
+							 NULL,
+							 NULL);
+
 	DefineCustomIntVariable(MAKE_EXTOPTION("max_open_chunks_per_insert"),
 							"Maximum open chunks per insert",
 							"Maximum number of open chunk tables per insert",
@@ -1334,28 +1272,6 @@ _guc_init(void)
 							   /* assign_hook= */ ts_license_guc_assign_hook,
 							   /* show_hook= */ NULL);
 
-	DefineCustomStringVariable(/* name= */ MAKE_EXTOPTION("last_tuned"),
-							   /* short_desc= */ "last tune run",
-							   /* long_desc= */ "records last time timescaledb-tune ran",
-							   /* valueAddr= */ &ts_last_tune_time,
-							   /* bootValue= */ NULL,
-							   /* context= */ PGC_SIGHUP,
-							   /* flags= */ 0,
-							   /* check_hook= */ NULL,
-							   /* assign_hook= */ NULL,
-							   /* show_hook= */ NULL);
-
-	DefineCustomStringVariable(/* name= */ MAKE_EXTOPTION("last_tuned_version"),
-							   /* short_desc= */ "version of timescaledb-tune",
-							   /* long_desc= */ "version of timescaledb-tune used to tune",
-							   /* valueAddr= */ &ts_last_tune_version,
-							   /* bootValue= */ NULL,
-							   /* context= */ PGC_SIGHUP,
-							   /* flags= */ 0,
-							   /* check_hook= */ NULL,
-							   /* assign_hook= */ NULL,
-							   /* show_hook= */ NULL);
-
 	DefineCustomEnumVariable(MAKE_EXTOPTION("bgw_log_level"),
 							 "Log level for the background worker subsystem",
 							 "Log level for the scheduler and workers of the background worker "
@@ -1512,16 +1428,6 @@ _guc_init(void)
 							 /* assign_hook= */ NULL,
 							 /* show_hook= */ NULL);
 
-	DefineCustomBoolVariable(/* name= */ MAKE_EXTOPTION("debug_allow_cagg_with_deprecated_funcs"),
-							 /* short_desc= */ "allow new caggs using time_bucket_ng",
-							 /* long_desc= */ "this is for debugging/testing purposes",
-							 /* valueAddr= */ &ts_guc_debug_allow_cagg_with_deprecated_funcs,
-							 /* bootValue= */ false,
-							 /* context= */ PGC_USERSET,
-							 /* flags= */ 0,
-							 /* check_hook= */ NULL,
-							 /* assign_hook= */ NULL,
-							 /* show_hook= */ NULL);
 #endif
 
 	/* register feature flags */
