@@ -146,7 +146,10 @@ ts_hypercube_add_slice(Hypercube *hc, const DimensionSlice *slice)
 void
 ts_hypercube_slice_sort(Hypercube *hc)
 {
-	qsort(hc->slices, hc->num_slices, sizeof(DimensionSlice *), cmp_slices_by_dimension_id);
+	qsort((void *) hc->slices,
+		  hc->num_slices,
+		  sizeof(DimensionSlice *),
+		  cmp_slices_by_dimension_id);
 }
 
 const DimensionSlice *
@@ -162,8 +165,8 @@ ts_hypercube_get_slice_by_dimension_id(const Hypercube *hc, int32 dimension_id)
 
 	Assert(hypercube_is_sorted(hc));
 
-	ptr = bsearch(&ptr,
-				  hc->slices,
+	ptr = bsearch((void *) &ptr,
+				  (void *) hc->slices,
 				  hc->num_slices,
 				  sizeof(DimensionSlice *),
 				  cmp_slices_by_dimension_id);
@@ -191,16 +194,10 @@ ts_hypercube_from_constraints(const ChunkConstraints *constraints, ScanIterator
 	for (i = 0; i < constraints->num_constraints; i++)
 	{
 		ChunkConstraint *cc = chunk_constraints_get(constraints, i);
-		ScanTupLock tuplock = {
-			.lockmode = LockTupleKeyShare,
-			.waitpolicy = LockWaitBlock,
-			.lockflags = TUPLE_LOCK_FLAG_FIND_LAST_VERSION,
-		};
 
 		if (is_dimension_constraint(cc))
 		{
 			DimensionSlice *slice;
-			ScanTupLock *const tuplock_ptr = RecoveryInProgress() ? NULL : &tuplock;
 
 			Assert(hc->num_slices < constraints->num_dimension_constraints);
 
@@ -213,10 +210,11 @@ ts_hypercube_from_constraints(const ChunkConstraints *constraints, ScanIterator
 			 * ephemeral recovery mode), so we only take the lock if we are not
 			 * in recovery mode.
 			 */
-			slice = ts_dimension_slice_scan_iterator_get_by_id(slice_it,
-															   cc->fd.dimension_slice_id,
-															   tuplock_ptr);
-			Assert(slice != NULL);
+			slice = ts_dimension_slice_scan_iterator_get_by_id(slice_it, cc->fd.dimension_slice_id);
+			if (!slice)
+			{
+				elog(ERROR, "could not find dimension slice with id %d", cc->fd.dimension_slice_id);
+			}
 			hc->slices[hc->num_slices++] = slice;
 		}
 	}
