@@ -35,6 +35,7 @@
 #include <optimizer/restrictinfo.h>
 #include <optimizer/tlist.h>
 #include <parser/parse_func.h>
+#include <parser/parse_type.h>
 #include <parser/parsetree.h>
 #include <partitioning/partbounds.h>
 #include <utils/builtins.h>
@@ -42,30 +43,31 @@
 #include <utils/errcodes.h>
 #include <utils/fmgroids.h>
 #include <utils/fmgrprotos.h>
+#include <utils/lsyscache.h>
 #include <utils/syscache.h>
+#include <utils/timestamp.h>
+#include <utils/typcache.h>
+#include <utils/uuid.h>
 
 #include "compat/compat.h"
+#include "annotations.h"
 #include "chunk.h"
 #include "cross_module_fn.h"
-#include "extension.h"
-#include "extension_constants.h"
 #include "guc.h"
+#include "hypercube.h"
 #include "hypertable.h"
 #include "hypertable_restrict_info.h"
 #include "import/planner.h"
 #include "nodes/chunk_append/chunk_append.h"
-#include "partialize.h"
-#include "partitioning.h"
 #include "planner.h"
 #include "time_utils.h"
-#include "ts_catalog/array_utils.h"
+#include "uuid.h"
 
 typedef struct CollectQualCtx
 {
 	PlannerInfo *root;
 	RelOptInfo *rel;
 	List *restrictions;
-	List *join_conditions;
 	List *propagate_conditions;
 	List *all_quals;
 	int join_level;
@@ -130,29 +132,64 @@ is_timestamptz_op_interval(Expr *expr)
 		   (c1->consttype == INTERVALOID && c2->consttype == TIMESTAMPTZOID);
 }
 
-static Datum
-int_get_datum(int64 value, Oid type)
+static Const *
+integral_timeval_to_const(int64 value, Oid type)
 {
+	bool typbyval = get_typbyval(type);
+
 	switch (type)
 	{
 		case INT2OID:
-			return Int16GetDatum(value);
+			return makeConst(type, -1, InvalidOid, 2, Int16GetDatum(value), false, typbyval);
 		case INT4OID:
-			return Int32GetDatum(value);
+			return makeConst(type, -1, InvalidOid, 4, Int32GetDatum(value), false, typbyval);
 		case INT8OID:
-			return Int64GetDatum(value);
+			return makeConst(type, -1, InvalidOid, 8, Int64GetDatum(value), false, typbyval);
+		case DATEOID:
+			return makeConst(type,
+							 -1,
+							 InvalidOid,
+							 sizeof(DateADT),
+							 DateADTGetDatum(value),
+							 false,
+							 typbyval);
 		case TIMESTAMPOID:
-			return TimestampGetDatum(value);
+			return makeConst(type,
+							 -1,
+							 InvalidOid,
+							 sizeof(Timestamp),
+							 TimestampGetDatum(value),
+							 false,
+							 typbyval);
 		case TIMESTAMPTZOID:
-			return TimestampTzGetDatum(value);
+			return makeConst(type,
+							 -1,
+							 InvalidOid,
+							 sizeof(TimestampTz),
+							 TimestampTzGetDatum(value),
+							 false,
+							 typbyval);
+		case UUIDOID:
+		{
+			/*
+			 * UUIDv7 doesn't support timestamps smaller than the UNIX epoch. However, caggs often
+			 * refresh from "beginning of time" so we need to restrict the lower boundary value to
+			 * the UNIX epoch.
+			 */
+			if (value <= UNIX_EPOCH_AS_TIMESTAMP)
+				value = UNIX_EPOCH_AS_TIMESTAMP;
+
+			pg_uuid_t *uuid = ts_create_uuid_v7_from_timestamptz((TimestampTz) value, true);
+			return makeConst(type, -1, InvalidOid, UUID_LEN, UUIDPGetDatum(uuid), false, typbyval);
+		}
 		default:
-			elog(ERROR, "unsupported datatype in int_get_datum: %s", format_type_be(type));
+			elog(ERROR, "unsupported datatype in %s: %s", __func__, format_type_be(type));
 			pg_unreachable();
 	}
 }
 
 static int64
-const_datum_get_int(Const *cnst)
+const_to_integral_timeval(const Const *cnst)
 {
 	Assert(!cnst->constisnull);
 
@@ -170,9 +207,19 @@ const_datum_get_int(Const *cnst)
 			return DatumGetTimestamp(cnst->constvalue);
 		case TIMESTAMPTZOID:
 			return DatumGetTimestampTz(cnst->constvalue);
+		case UUIDOID:
+		{
+			/*
+			 * While it is possible to extract the timestamp from a UUID, there is currently no use
+			 * case where this function is used since the UUID-based time_bucket() function returns
+			 * a timestamptz. Thus, any value compared to is also a timestamptz and not a UUID.
+			 * */
+			TS_FALLTHROUGH;
+		}
 		default:
 			elog(ERROR,
-				 "unsupported datatype in const_datum_get_int: %s",
+				 "unsupported datatype in %s: %s",
+				 __func__,
 				 format_type_be(cnst->consttype));
 			pg_unreachable();
 	}
@@ -329,49 +376,177 @@ constify_timestamptz_op_interval(PlannerInfo *root, OpExpr *constraint)
 									constraint->inputcollid);
 }
 
+typedef struct TimeBucketInfo
+{
+	Oid rettype;	/* Type of the return value */
+	Const *width;	/* Bucket width */
+	Node *timeval;	/* Bucket "time" value */
+	Oid timetype;	/* Type of the time value */
+	uint16 numargs; /* Number of bucket function arguments */
+} TimeBucketInfo;
+
+/*
+ * Representation of a parse time bucket Qual:
+ *
+ *  <time_bucket() OP value>
+ */
+typedef struct TimeBucketQual
+{
+	TimeBucketInfo tb;
+	int strategy;
+	Const *value;
+} TimeBucketQual;
+
+/*
+ * Parse an expression of form <time_bucket(width, column) OP value> and extract the important
+ * components into a TimeBucketQual struct.
+ *
+ * Returns false if the expression does not fit the expected format.
+ */
 static bool
-extract_opexpr_parts(Expr *node, OpExpr **op, FuncExpr **time_bucket, Expr **value, Oid *opno)
+extract_time_bucket_qual(Expr *node, TimeBucketQual *tbqual)
 {
 	if (!IsA(node, OpExpr))
-	{
 		return false;
-	}
 
-	*op = castNode(OpExpr, node);
-	if (list_length((*op)->args) != 2)
-	{
+	OpExpr *op = castNode(OpExpr, node);
+
+	if (list_length((op)->args) != 2)
 		return false;
-	}
 
-	Expr *left = linitial((*op)->args);
-	Expr *right = lsecond((*op)->args);
+	Expr *left = linitial((op)->args);
+	Expr *right = lsecond((op)->args);
+	FuncExpr *time_bucket;
+
+	MemSet(tbqual, 0, sizeof(TimeBucketQual));
+
+	Oid opno = InvalidOid;
 
 	if (IsA(left, FuncExpr) && IsA(right, Const))
 	{
-		*time_bucket = castNode(FuncExpr, left);
-		*value = right;
-		*opno = (*op)->opno;
+		time_bucket = castNode(FuncExpr, left);
+		tbqual->value = castNode(Const, right);
+		opno = op->opno;
 	}
-	else if (IsA(right, FuncExpr))
+	else if (IsA(right, FuncExpr) && IsA(left, Const))
 	{
-		*time_bucket = castNode(FuncExpr, right);
-		*value = left;
-		*opno = get_commutator((*op)->opno);
-
-		if (!OidIsValid(opno))
-			return false;
+		time_bucket = castNode(FuncExpr, right);
+		tbqual->value = castNode(Const, left);
+		opno = get_commutator(op->opno);
 	}
 	else
 	{
 		return false;
 	}
 
-	if (!is_time_bucket_function((Expr *) *time_bucket) || !IsA(*value, Const) ||
-		castNode(Const, *value)->constisnull)
-	{
+	if (!is_time_bucket_function((Expr *) time_bucket) || tbqual->value->constisnull)
 		return false;
-	}
 
+	Const *width = linitial(time_bucket->args);
+	/* Get the time/partitioning column argument */
+	Node *timearg = lsecond(time_bucket->args);
+
+	if (!IsA(width, Const) || width->constisnull)
+		return false;
+
+	tbqual->tb.numargs = list_length(time_bucket->args);
+	tbqual->tb.width = width;
+	tbqual->tb.timeval = timearg;
+	tbqual->tb.timetype = exprType(timearg);
+	tbqual->tb.rettype = exprType((Node *) time_bucket);
+
+	/* 3 or more args should have Const 3rd arg */
+	if (list_length(time_bucket->args) > 2 && !IsA(lthird(time_bucket->args), Const))
+		return false;
+
+	/* 5 args variants should have Const 4th and 5th arg */
+	if (list_length(time_bucket->args) == 5 &&
+		(!IsA(lfourth(time_bucket->args), Const) || !IsA(lfifth(time_bucket->args), Const)))
+		return false;
+
+	Assert(list_length(time_bucket->args) == 2 || list_length(time_bucket->args) == 3 ||
+		   list_length(time_bucket->args) == 5);
+
+	TypeCacheEntry *tce = lookup_type_cache(tbqual->tb.rettype, TYPECACHE_BTREE_OPFAMILY);
+	tbqual->strategy = get_op_opfamily_strategy(opno, tce->btree_opf);
+
+	return true;
+}
+
+/*
+ * Convert at time_bucket() width argument (typically Interval or integer) to a microseconds
+ * integer. Also check that the width (interval) doesn't overflow the time value.
+ */
+static bool
+time_bucket_width_to_integral(const Const *width, Oid bucket_type, int64 integral_value,
+							  int64 *integral_width)
+{
+	switch (width->consttype)
+	{
+		case INT2OID:
+		case INT4OID:
+		case INT8OID:
+
+			/* We can support the offset variants of time_bucket as the
+			 * amount of shifting they do is never bigger than the bucketing
+			 * width.
+			 */
+			*integral_width = const_to_integral_timeval(width);
+
+			if (integral_value >= ts_time_get_max(bucket_type) - *integral_width)
+				return false;
+			break;
+		case INTERVALOID:
+		{
+			Interval *interval = DatumGetIntervalP(width->constvalue);
+			/*
+			 * Optimization can't be applied when interval has month component.
+			 */
+			if (interval->month != 0)
+				return false;
+
+			if (bucket_type == DATEOID)
+			{
+				/* bail out if interval->time can't be exactly represented as a double */
+				if (interval->time >= 0x3FFFFFFFFFFFFFLL)
+					return false;
+
+				*integral_width =
+					interval->day + ceil((double) interval->time / (double) USECS_PER_DAY);
+
+				if (integral_value >= (TS_DATE_END - *integral_width))
+					return false;
+			}
+			else if (bucket_type == TIMESTAMPOID || bucket_type == TIMESTAMPTZOID)
+			{
+				/*
+				 * If width interval has day component we merge it with time component
+				 */
+				*integral_width = interval->time;
+
+				if (interval->day != 0)
+				{
+					/*
+					 * if our transformed restriction would overflow we skip adding it
+					 */
+					if (interval->time >= TS_TIMESTAMP_END - interval->day * USECS_PER_DAY)
+						return false;
+
+					*integral_width += interval->day * USECS_PER_DAY;
+				}
+
+				if (integral_value >= (TS_TIMESTAMP_END - *integral_width))
+					return false;
+			}
+			else
+			{
+				return false;
+			}
+			break;
+		}
+		default:
+			return false;
+	}
 	return true;
 }
 
@@ -412,239 +587,88 @@ extract_opexpr_parts(Expr *node, OpExpr **op, FuncExpr **time_bucket, Expr **val
 Expr *
 ts_transform_time_bucket_comparison(Expr *node)
 {
-	FuncExpr *time_bucket;
-	Expr *value;
-	OpExpr *op;
-	Oid opno;
+	TimeBucketQual tbqual;
 
-	if (!extract_opexpr_parts(node, &op, &time_bucket, &value, &opno))
+	if (!extract_time_bucket_qual(node, &tbqual))
+		return NULL;
+
+	/*
+	 * The qual is an expression <time_bucket OP value> or <value OP time_bucket>. Convert the value
+	 * to integral time format.
+	 */
+	int64 integral_value = const_to_integral_timeval(tbqual.value);
+	Const *newvalue = NULL;
+
+	/*
+	 * We strip the time_bucket() from the expression, leaving the input "time" argument. Depending
+	 * on the comparison OP, the value might need adjustment. Then the value is converted to the
+	 * input/column type for time_bucket(). In most cases, the value's original type and the bucket
+	 * input type is the same (e.g. TIMESTAMPTZ), but in some cases they differ. For example, it is
+	 * possible to compare an int8 bucket function with an int4 value. In the case of UUID bucket,
+	 * the bucket function's input type (UUID) is different from the output type (TIMESTAMPTZ), so
+	 * the timestamp value needs to be converted to a boundary UUID.
+	 */
+	switch (tbqual.strategy)
 	{
-		return NULL;
-	}
-
-	Const *width = linitial(time_bucket->args);
-
-	if (!IsA(width, Const) || width->constisnull)
-		return NULL;
-
-	/* 3 or more args should have Const 3rd arg */
-	if (list_length(time_bucket->args) > 2 && !IsA(lthird(time_bucket->args), Const))
-		return NULL;
-
-	/* 5 args variants should have Const 4th and 5th arg */
-	if (list_length(time_bucket->args) == 5 &&
-		(!IsA(lfourth(time_bucket->args), Const) || !IsA(lfifth(time_bucket->args), Const)))
-		return NULL;
-
-	Assert(list_length(time_bucket->args) == 2 || list_length(time_bucket->args) == 3 ||
-		   list_length(time_bucket->args) == 5);
-
-	TypeCacheEntry *tce;
-	int strategy;
-
-	tce = lookup_type_cache(exprType((Node *) time_bucket), TYPECACHE_BTREE_OPFAMILY);
-	strategy = get_op_opfamily_strategy(opno, tce->btree_opf);
-
-	if (strategy == BTGreaterStrategyNumber || strategy == BTGreaterEqualStrategyNumber)
-	{
-		/* Since time_bucket will always shift the input to the left this
-		 * transformation is always safe even in the presence of offset variants.
-		 *
-		 * column > value
-		 */
-		op = copyObject(op);
-		op->args = list_make2(lsecond(time_bucket->args), value);
-
-		/*
-		 * if we switched operator we need to adjust OpExpr as well
-		 */
-		if (op->opno != opno)
+		case BTGreaterStrategyNumber:
+		case BTGreaterEqualStrategyNumber:
+			/*
+			 * Since time_bucket will always shift the input to the left this
+			 * transformation is always safe even in the presence of offset variants.
+			 *
+			 * Handle expressions of form:
+			 *
+			 *  - column > value
+			 *  - column >= value
+			 */
+			newvalue = integral_timeval_to_const(integral_value, tbqual.tb.timetype);
+			break;
+		case BTLessStrategyNumber:
+		case BTLessEqualStrategyNumber:
 		{
-			op->opno = opno;
-			op->opfuncid = InvalidOid;
-		}
+			/*
+			 * Handle expressions of form:
+			 *
+			 *  - column < value + width
+			 *  - column <= value + width
+			 *  */
+			int64 integral_width = 0;
 
-		return &op->xpr;
-	}
-	else if (strategy == BTLessStrategyNumber || strategy == BTLessEqualStrategyNumber)
-	{
-		/* column < value + width */
-		Expr *subst;
-		Datum datum;
-		int64 integralValue, integralWidth;
-
-		switch (tce->type_id)
-		{
-			case INT2OID:
-			case INT4OID:
-			case INT8OID:
-				/* We can support the offset variants of time_bucket as the
-				 * amount of shifting they do is never bigger than the bucketing
-				 * width.
-				 */
-				integralValue = const_datum_get_int(castNode(Const, value));
-				integralWidth = const_datum_get_int(width);
-
-				if (integralValue >= ts_time_get_max(tce->type_id) - integralWidth)
-					return NULL;
-
-				/*
-				 * When the time_bucket constraint matches the start of the bucket
-				 * and we have a less than constraint and no offset  we can skip
-				 * adding the full bucket.
-				 */
-				if (strategy == BTLessStrategyNumber && list_length(time_bucket->args) == 2 &&
-					integralValue % integralWidth == 0)
-					datum = int_get_datum(integralValue, tce->type_id);
-				else
-					datum = int_get_datum(integralValue + integralWidth, tce->type_id);
-
-				subst = (Expr *) makeConst(tce->type_id,
-										   -1,
-										   InvalidOid,
-										   tce->typlen,
-										   datum,
-										   false,
-										   tce->typbyval);
-				break;
-
-			case DATEOID:
-			{
-				/* We can support the offset/origin variants of time_bucket
-				 * as the amount of shifting they do is never bigger than the
-				 * bucketing width.
-				 */
-				Assert(width->consttype == INTERVALOID);
-				Interval *interval = DatumGetIntervalP(width->constvalue);
-
-				/*
-				 * Optimization can't be applied when interval has month component.
-				 */
-				if (interval->month != 0)
-					return NULL;
-
-				/* bail out if interval->time can't be exactly represented as a double */
-				if (interval->time >= 0x3FFFFFFFFFFFFFLL)
-					return NULL;
-
-				integralValue = const_datum_get_int(castNode(Const, value));
-				integralWidth =
-					interval->day + ceil((double) interval->time / (double) USECS_PER_DAY);
-
-				if (integralValue >= (TS_DATE_END - integralWidth))
-					return NULL;
-
-				/*
-				 * When the time_bucket constraint matches the start of the bucket
-				 * and we have a less than constraint and no offset or origin we can
-				 * skip adding the full bucket.
-				 */
-				if (strategy == BTLessStrategyNumber && list_length(time_bucket->args) == 2 &&
-					integralValue % integralWidth == 0)
-					datum = DateADTGetDatum(integralValue);
-				else
-					datum = DateADTGetDatum(integralValue + integralWidth);
-
-				subst = (Expr *) makeConst(tce->type_id,
-										   -1,
-										   InvalidOid,
-										   tce->typlen,
-										   datum,
-										   false,
-										   tce->typbyval);
-
-				break;
-			}
-			case TIMESTAMPOID:
-			case TIMESTAMPTZOID:
-			{
-				/* We can support the offset/origin/timezone variants of time_bucket
-				 * as the amount of shifting they do is never bigger than the
-				 * bucketing width.
-				 */
-				Assert(width->consttype == INTERVALOID);
-				Interval *interval = DatumGetIntervalP(width->constvalue);
-
-				/*
-				 * Optimization can't be applied when interval has month component.
-				 */
-				if (interval->month != 0)
-					return NULL;
-
-				/*
-				 * If width interval has day component we merge it with time component
-				 */
-
-				integralWidth = interval->time;
-				if (interval->day != 0)
-				{
-					/*
-					 * if our transformed restriction would overflow we skip adding it
-					 */
-					if (interval->time >= TS_TIMESTAMP_END - interval->day * USECS_PER_DAY)
-						return NULL;
-
-					integralWidth += interval->day * USECS_PER_DAY;
-				}
-
-				integralValue = const_datum_get_int(castNode(Const, value));
-
-				if (integralValue >= (TS_TIMESTAMP_END - integralWidth))
-					return NULL;
-
-				/*
-				 * When the time_bucket constraint matches the start of the bucket
-				 * and we have a less than constraint and no other modifying arguments
-				 * we can skip adding the full bucket.
-				 */
-				if (strategy == BTLessStrategyNumber && list_length(time_bucket->args) == 2 &&
-					integralValue % integralWidth == 0)
-					datum = int_get_datum(integralValue, tce->type_id);
-				else
-					datum = int_get_datum(integralValue + integralWidth, tce->type_id);
-
-				subst = (Expr *) makeConst(tce->type_id,
-										   -1,
-										   InvalidOid,
-										   tce->typlen,
-										   datum,
-										   false,
-										   tce->typbyval);
-
-				break;
-			}
-
-			default:
+			if (!time_bucket_width_to_integral(tbqual.tb.width,
+											   tbqual.tb.rettype,
+											   integral_value,
+											   &integral_width))
 				return NULL;
+
+			if (tbqual.strategy == BTLessStrategyNumber && tbqual.tb.numargs == 2 &&
+				integral_value % integral_width == 0)
+				newvalue = integral_timeval_to_const(integral_value, tbqual.tb.timetype);
+			else
+				newvalue =
+					integral_timeval_to_const(integral_value + integral_width, tbqual.tb.timetype);
+
+			break;
 		}
-
-		/*
-		 * adjust toplevel expression if datatypes changed
-		 * this can happen when comparing int4 values against int8 time_bucket
-		 */
-		if (tce->type_id != castNode(Const, value)->consttype)
-		{
-			opno =
-				ts_get_operator(get_opname(opno), PG_CATALOG_NAMESPACE, tce->type_id, tce->type_id);
-
-			if (!OidIsValid(opno))
-				return NULL;
-		}
-
-		op = copyObject(op);
-
-		/*
-		 * if we changed operator we need to adjust OpExpr as well
-		 */
-		if (op->opno != opno)
-		{
-			op->opno = opno;
-			op->opfuncid = get_opcode(opno);
-		}
-
-		op->args = list_make2(lsecond(time_bucket->args), subst);
+		default:
+			return NULL;
 	}
 
+	Assert(newvalue != NULL);
+
+	/* Create a new "unwrapped" OpExpr using the time_bucket() input/column type */
+	TypeCacheEntry *tce = lookup_type_cache(tbqual.tb.timetype, TYPECACHE_BTREE_OPFAMILY);
+	Oid opno = get_opfamily_member(tce->btree_opf,
+								   tce->btree_opintype,
+								   tce->btree_opintype,
+								   tbqual.strategy);
+
+	OpExpr *op = (OpExpr *) copyObject(node);
+	op->args = list_make2(tbqual.tb.timeval, newvalue);
+	op->opno = opno;
+
+	/* The operator might have changed, so reset the function ID */
+	op->opfuncid = InvalidOid;
+
 	return &op->xpr;
 }
 
@@ -654,6 +678,12 @@ ts_transform_time_bucket_comparison(Expr *node)
  * will just result in more chunks being included) so this does not need
  * to be as comprehensive as the PG native derivation. This is inspired
  * by the derivation in `deconstruct_recurse` in PG
+ *
+ * TODO: as of 2025, the baserestrictinfo and joininfo is already set when the
+ * hypertable expansion code is called, so this does duplicate work. If any bugs
+ * are found in this code, it should be switched to use the RelOptInfos and
+ * equivalence classes instead of the parse tree. The chunk exclusion code for
+ * the non-join clauses was already changed to use the former.
  */
 static Node *
 process_quals(Node *quals, CollectQualCtx *ctx, bool is_outer_join)
@@ -763,13 +793,7 @@ timebucket_annotate(Node *quals, CollectQualCtx *ctx)
 /*
  * collect JOIN information
  *
- * This function adds information to two lists in the CollectQualCtx
- *
- * join_conditions
- *
- * This list contains all equality join conditions and is used by
- * ChunkAppend to decide whether the ordered append optimization
- * can be applied.
+ * This function adds information to the CollectQualCtx
  *
  * propagate_conditions
  *
@@ -814,8 +838,6 @@ collect_join_quals(Node *quals, CollectQualCtx *ctx, bool can_propagate)
 
 				if (op->opno == tce->eq_opr)
 				{
-					ctx->join_conditions = lappend(ctx->join_conditions, op);
-
 					if (can_propagate)
 						ctx->propagate_conditions = lappend(ctx->propagate_conditions, op);
 				}
@@ -892,8 +914,8 @@ find_children_chunks(HypertableRestrictInfo *hri, Hypertable *ht, bool include_o
 }
 
 static bool
-should_order_append(PlannerInfo *root, RelOptInfo *rel, Hypertable *ht, List *join_conditions,
-					int *order_attno, bool *reverse)
+should_order_append(PlannerInfo *root, RelOptInfo *rel, Hypertable *ht, int *order_attno,
+					bool *reverse)
 {
 	/* check if optimizations are enabled */
 	if (!ts_guc_enable_optimizations || !ts_guc_enable_ordered_append ||
@@ -907,7 +929,65 @@ should_order_append(PlannerInfo *root, RelOptInfo *rel, Hypertable *ht, List *jo
 	if (root->parse->sortClause == NIL)
 		return false;
 
-	return ts_ordered_append_should_optimize(root, rel, ht, join_conditions, order_attno, reverse);
+	return ts_ordered_append_should_optimize(root, rel, ht, order_attno, reverse);
+}
+
+/*
+ * Some time conditions are not directly applicable for the chunk exclusion, but
+ * imply a simpler time comparison condition which can be used for hypertable
+ * expansion. Return a list of any simplified restrictions we could build for
+ * the restrictions in the given list.
+ */
+static List *
+get_simplified_restrictions(PlannerInfo *root, List *restrictions)
+{
+	List *simplified_restrictions = NIL;
+	ListCell *lc;
+	foreach (lc, restrictions)
+	{
+		RestrictInfo *ri = castNode(RestrictInfo, lfirst(lc));
+		Expr *qual = ri->clause;
+		if (IsA(qual, OpExpr) && list_length(castNode(OpExpr, qual)->args) == 2)
+		{
+			OpExpr *op = castNode(OpExpr, qual);
+			Expr *left = linitial(op->args);
+			Expr *right = lsecond(op->args);
+
+			if ((IsA(left, Var) && is_timestamptz_op_interval(right)) ||
+				(IsA(right, Var) && is_timestamptz_op_interval(left)))
+			{
+				/*
+				 * Check for constraints with TIMESTAMPTZ OP INTERVAL calculations.
+				 */
+				Expr *transformed = (Expr *) constify_timestamptz_op_interval(root, op);
+				if (transformed != (Expr *) op)
+				{
+					RestrictInfo *ri_copy = copyObject(ri);
+					ri_copy->clause = transformed;
+					simplified_restrictions = lappend(simplified_restrictions, ri_copy);
+				}
+			}
+			else
+			{
+				/*
+				 * check for time_bucket comparisons
+				 * time_bucket(Const, time_colum) > Const
+				 */
+				Expr *transformed = ts_transform_time_bucket_comparison(qual);
+				if (transformed != NULL)
+				{
+					/*
+					 * Also use the transformed qual for chunk exclusion.
+					 */
+					RestrictInfo *ri_copy = copyObject(ri);
+					ri_copy->clause = transformed;
+					simplified_restrictions = lappend(simplified_restrictions, ri_copy);
+				}
+			}
+		}
+	}
+
+	return simplified_restrictions;
 }
 
 /**
@@ -920,8 +1000,8 @@ should_order_append(PlannerInfo *root, RelOptInfo *rel, Hypertable *ht, List *jo
  * on rel->fdw_private when appends are ordered.
  */
 static Chunk **
-get_chunks(CollectQualCtx *ctx, PlannerInfo *root, RelOptInfo *rel, Hypertable *ht,
-		   bool include_osm, unsigned int *num_chunks)
+get_chunks(PlannerInfo *root, RelOptInfo *rel, Hypertable *ht, bool include_osm,
+		   unsigned int *num_chunks, HypertableRestrictInfo **hri_out)
 {
 	bool reverse;
 	int order_attno;
@@ -931,9 +1011,19 @@ get_chunks(CollectQualCtx *ctx, PlannerInfo *root, RelOptInfo *rel, Hypertable *
 	/*
 	 * This is where the magic happens: use our HypertableRestrictInfo
 	 * infrastructure to deduce the appropriate chunks using our range
-	 * exclusion
+	 * exclusion.
 	 */
-	ts_hypertable_restrict_info_add(hri, root, ctx->restrictions);
+	ts_hypertable_restrict_info_add(hri, root, rel->baserestrictinfo);
+
+	List *simplified_restrictions = get_simplified_restrictions(root, rel->baserestrictinfo);
+	ts_hypertable_restrict_info_add(hri, root, simplified_restrictions);
+
+	/* Limit to hypertables without multiple dimensions for now */
+	if (hri->num_base_restrictions >= 1 && hri->num_dimensions == 1 &&
+		ht->space->num_dimensions == 1)
+	{
+		*hri_out = hri;
+	}
 
 	/*
 	 * If fdw_private has not been setup by caller there is no point checking
@@ -941,8 +1031,7 @@ get_chunks(CollectQualCtx *ctx, PlannerInfo *root, RelOptInfo *rel, Hypertable *
 	 * to signal that this is safe to transform in ordered append plan in
 	 * set_rel_pathlist.
 	 */
-	if (rel->fdw_private != NULL &&
-		should_order_append(root, rel, ht, ctx->join_conditions, &order_attno, &reverse))
+	if (rel->fdw_private != NULL && should_order_append(root, rel, ht, &order_attno, &reverse))
 	{
 		TimescaleDBPrivate *priv = ts_get_private_reloptinfo(rel);
 		List **nested_oids = NULL;
@@ -997,7 +1086,6 @@ ts_plan_expand_timebucket_annotate(PlannerInfo *root, RelOptInfo *rel)
 		.rel = rel,
 		.restrictions = NIL,
 		.all_quals = NIL,
-		.join_conditions = NIL,
 		.propagate_conditions = NIL,
 	};
 
@@ -1008,6 +1096,102 @@ ts_plan_expand_timebucket_annotate(PlannerInfo *root, RelOptInfo *rel)
 		propagate_join_quals(root, rel, &ctx);
 }
 
+/*
+ * Build a list of baserestrictinfo with any Var OP Const constraints on the primary
+ * dimension removed.
+ */
+static List *
+filter_baserestrictions(Hypertable *ht, List *base_restrictions)
+{
+	AttrNumber dim_attno = ht->space->dimensions[0].column_attno;
+	List *filtered_restrictions = NIL;
+	ListCell *lc;
+	foreach (lc, base_restrictions)
+	{
+		RestrictInfo *ri = castNode(RestrictInfo, lfirst(lc));
+		Expr *qual = ri->clause;
+		if (IsA(qual, OpExpr))
+		{
+			OpExpr *op = castNode(OpExpr, qual);
+			Node *left = strip_implicit_coercions(linitial(op->args));
+			Node *right = strip_implicit_coercions(lsecond(op->args));
+			if ((IsA(left, Var) && IsA(right, Const) &&
+				 castNode(Var, left)->varattno == dim_attno) ||
+				(IsA(right, Var) && IsA(left, Const) &&
+				 castNode(Var, right)->varattno == dim_attno))
+			{
+				/* only consider simple column to constant comparisons */
+				continue;
+			}
+		}
+
+		filtered_restrictions = lappend(filtered_restrictions, ri);
+	}
+	return filtered_restrictions;
+}
+
+/*
+ * Returns true if the given chunk is fully included by the restrictions
+ * on the primary dimension.
+ */
+static bool
+chunk_fully_covered(HypertableRestrictInfo *hri, Chunk *chunk)
+{
+	DimensionRestrictInfoOpen *dri = (DimensionRestrictInfoOpen *) hri->dimension_restriction[0];
+	Ensure(dri->base.dimension->type == DIMENSION_TYPE_OPEN, "primary dimension must be open");
+	Ensure(hri->num_base_restrictions > 0, "must have base restrictions");
+
+	if (IS_OSM_CHUNK(chunk) ||
+		(dri->lower_strategy == InvalidStrategy && dri->upper_strategy == InvalidStrategy) ||
+		(chunk->cube->slices[0]->fd.range_start == TS_TIME_NOBEGIN ||
+		 chunk->cube->slices[0]->fd.range_end == TS_TIME_NOEND))
+		return false;
+
+	/*
+	 * DimensionRetrictInfo strategy should only be one BTGreaterStrategyNumber
+	 * or BTGreaterEqualStrategyNumber on the lower boundary and
+	 * BTLessStrategyNumber or BTLessEqualStrategyNumber on the upper boundary.
+	 *
+	 * BTEqualStrategyNumber gets changed to BTGreaterEqualStrategyNumber
+	 * on lower boundary and BTLessEqualStrategyNumber on upper boundary.
+	 */
+	if (dri->lower_strategy != InvalidStrategy)
+	{
+		switch (dri->lower_strategy)
+		{
+			case BTGreaterStrategyNumber:
+				if (chunk->cube->slices[0]->fd.range_start <= dri->lower_bound)
+					return false;
+				break;
+			case BTGreaterEqualStrategyNumber:
+				if (chunk->cube->slices[0]->fd.range_start < dri->lower_bound)
+					return false;
+				break;
+			default:
+				/* Should never happen */
+				elog(ERROR, "unexpected dimension restrictinfo strategy: %d", dri->upper_strategy);
+		}
+	}
+	if (dri->upper_strategy != InvalidStrategy)
+	{
+		switch (dri->upper_strategy)
+		{
+			case BTLessStrategyNumber:
+				if (chunk->cube->slices[0]->fd.range_end > dri->upper_bound)
+					return false;
+				break;
+			case BTLessEqualStrategyNumber:
+				if (chunk->cube->slices[0]->fd.range_end - 1 > dri->upper_bound)
+					return false;
+				break;
+			default:
+				/* Should never happen */
+				elog(ERROR, "unexpected dimension restrictinfo strategy: %d", dri->upper_strategy);
+		}
+	}
+	return true;
+}
+
 /* Inspired by expand_inherited_rtentry but expands
  * a hypertable chunks into an append relation. */
 void
@@ -1025,7 +1209,6 @@ ts_plan_expand_hypertable_chunks(Hypertable *ht, PlannerInfo *root, RelOptInfo *
 		.rel = rel,
 		.restrictions = NIL,
 		.all_quals = NIL,
-		.join_conditions = NIL,
 		.propagate_conditions = NIL,
 		.join_level = 0,
 	};
@@ -1044,7 +1227,9 @@ ts_plan_expand_hypertable_chunks(Hypertable *ht, PlannerInfo *root, RelOptInfo *
 
 	Chunk **chunks = NULL;
 	unsigned int num_chunks = 0;
-	chunks = get_chunks(&ctx, root, rel, ht, include_osm, &num_chunks);
+
+	HypertableRestrictInfo *hri = NULL;
+	chunks = get_chunks(root, rel, ht, include_osm, &num_chunks, &hri);
 	/* Can have zero chunks. */
 	Assert(num_chunks == 0 || chunks != NULL);
 
@@ -1137,10 +1322,7 @@ ts_plan_expand_hypertable_chunks(Hypertable *ht, PlannerInfo *root, RelOptInfo *
 		appinfo->child_relid = child_rtindex;
 		appinfo->parent_reltype = oldrelation->rd_rel->reltype;
 		appinfo->child_reltype = newrelation->rd_rel->reltype;
-		ts_make_inh_translation_list(oldrelation,
-									 newrelation,
-									 child_rtindex,
-									 &appinfo->translated_vars);
+		ts_make_inh_translation_list(oldrelation, newrelation, child_rtindex, appinfo);
 		appinfo->parent_reloid = parent_oid;
 		appinfos = lappend(appinfos, appinfo);
 
@@ -1158,17 +1340,47 @@ ts_plan_expand_hypertable_chunks(Hypertable *ht, PlannerInfo *root, RelOptInfo *
 	 * build_simple_rel will look things up in the append_rel_array, so we can
 	 * only use it after that array has been set up.
 	 */
+	List *base_restrictions = rel->baserestrictinfo;
+	List *filtered_restrictions = NIL;
+	bool try_restriction_filtering =
+		ts_guc_enable_qual_filtering && hri && ht->space->num_dimensions == 1;
+
+	if (try_restriction_filtering)
+	{
+		filtered_restrictions = filter_baserestrictions(ht, base_restrictions);
+		/* Dont try filtering if all restrictions remain after filtering */
+		if (list_length(base_restrictions) == list_length(filtered_restrictions))
+			try_restriction_filtering = false;
+	}
+
 	for (unsigned int i = 0; i < num_chunks; i++)
 	{
+		bool can_clear_restrictinfo = false;
 		Index child_rtindex = first_chunk_index + i;
+		Chunk *chunk = chunks[i];
+		if (try_restriction_filtering)
+		{
+			can_clear_restrictinfo = chunk_fully_covered(hri, chunk);
+		}
+
+		/* build_simple_rel will copy baserestrictinfo to the child rel and
+		 * do the necessary attribute mapping. If we can determine that the chunk
+		 * is fully covered by the primary dimension restriction we can remove
+		 * primary dimension restrictions from baserestrictinfo.
+		 */
+		if (can_clear_restrictinfo)
+			rel->baserestrictinfo = filtered_restrictions;
+
 		/* build_simple_rel will add the child to the relarray */
 		RelOptInfo *child_rel = build_simple_rel(root, child_rtindex, rel);
 
+		if (can_clear_restrictinfo)
+			rel->baserestrictinfo = base_restrictions;
+
 		/*
 		 * Can't touch fdw_private for OSM chunks, it might be managed by the
 		 * OSM extension, or, in the tests, by postgres_fdw.
 		 */
-		Chunk *chunk = chunks[i];
 		if (!IS_OSM_CHUNK(chunk))
 		{
 			Assert(chunk->table_id == root->simple_rte_array[child_rtindex]->relid);
@@ -1205,7 +1417,7 @@ propagate_join_quals(PlannerInfo *root, RelOptInfo *rel, CollectQualCtx *ctx)
 		Var *rel_var, *other_var;
 
 		/*
-		 * join_conditions only has OpExpr with 2 Var as arguments
+		 * propagate_conditions only has OpExpr with 2 Var as arguments
 		 * this is enforced in process_quals
 		 */
 		Assert(IsA(op, OpExpr) && list_length(castNode(OpExpr, op)->args) == 2);
