@@ -54,7 +54,6 @@
 #include "nodes/constraint_aware_append/constraint_aware_append.h"
 #include "nodes/modify_hypertable.h"
 #include "partitioning.h"
-#include "planner/partialize.h"
 #include "planner/planner.h"
 #include "sort_transform.h"
 #include "utils.h"
@@ -461,7 +460,8 @@ preprocess_query(Node *node, PreprocessQueryContext *context)
 						 * We want to distinguish between the two cases here by
 						 * marking the chunk when rte->inh is true.
 						 */
-						Chunk *chunk = ts_chunk_get_by_relid(rte->relid, false);
+						Chunk *chunk =
+							ts_chunk_get_by_relid_locked(rte->relid, NoLock, NULL, false);
 						if (chunk && rte->inh)
 							rte_mark_for_expansion(rte);
 					}
@@ -698,25 +698,6 @@ timescaledb_planner(Query *parse, const char *query_string, int cursor_opts,
 					ts_modify_hypertable_fixup_tlist(subplan);
 			}
 
-			if (IsA(stmt->planTree, Agg))
-			{
-				Agg *agg = castNode(Agg, stmt->planTree);
-
-				/* If top-level plan is the finalize step of a partial
-				 * aggregation, and it is wrapped in the partialize_agg()
-				 * function, we want to do the combine step but skip
-				 * finalization (e.g., for avg(), add up individual
-				 * sum+counts, but don't compute the final average). */
-				if (agg->aggsplit == AGGSPLIT_FINAL_DESERIAL &&
-					has_partialize_function((Node *) agg->plan.targetlist, TS_FIX_AGGSPLIT_FINAL))
-				{
-					/* Deserialize input -> combine -> skip the final step ->
-					 * serialize again */
-					agg->aggsplit = AGGSPLITOP_COMBINE | AGGSPLITOP_DESERIALIZE |
-									AGGSPLITOP_SERIALIZE | AGGSPLITOP_SKIPFINAL;
-				}
-			}
-
 			ts_cm_functions->tsl_postprocess_plan(stmt);
 		}
 
@@ -1074,7 +1055,7 @@ should_chunk_append(Hypertable *ht, PlannerInfo *root, RelOptInfo *rel, Path *pa
 					FuncInfo *info = ts_func_cache_get_bucketing_func(func->funcid);
 					Expr *transformed;
 
-					if (info != NULL)
+					if (info && info->sort_transform)
 					{
 						transformed = info->sort_transform(func);
 						if (IsA(transformed, Var) &&
@@ -1494,14 +1475,14 @@ timescaledb_get_relation_info_hook(PlannerInfo *root, Oid relation_objectid, boo
 			 * based on the Chunk struct cached by our hypertable expansion, but
 			 * in cases when these functions don't run, we have to do it here.
 			 */
-			const bool use_transparent_decompression =
-				ts_guc_enable_transparent_decompression && TS_HYPERTABLE_HAS_COMPRESSION_TABLE(ht);
+			const bool use_columnar_scan =
+				ts_guc_enable_columnarscan && TS_HYPERTABLE_HAS_COMPRESSION_TABLE(ht);
 			const bool is_standalone_chunk = (type == TS_REL_CHUNK_STANDALONE) &&
 											 !TS_HYPERTABLE_IS_INTERNAL_COMPRESSION_TABLE(ht);
 			const bool is_child_chunk_in_update =
 				(type == TS_REL_CHUNK_CHILD) && IS_UPDL_CMD(query);
 
-			if (use_transparent_decompression && (is_standalone_chunk || is_child_chunk_in_update))
+			if (use_columnar_scan && (is_standalone_chunk || is_child_chunk_in_update))
 			{
 				const Chunk *chunk = ts_planner_chunk_fetch(root, rel);
 
@@ -1696,7 +1677,6 @@ timescaledb_create_upper_paths_hook(PlannerInfo *root, UpperRelationKind stage,
 									RelOptInfo *input_rel, RelOptInfo *output_rel, void *extra)
 {
 	Query *parse = root->parse;
-	bool partials_found = false;
 	TsRelType reltype = TS_REL_OTHER;
 	Hypertable *ht = NULL;
 
@@ -1715,14 +1695,6 @@ timescaledb_create_upper_paths_hook(PlannerInfo *root, UpperRelationKind stage,
 		if (output_rel->pathlist != NIL)
 			output_rel->pathlist =
 				replace_modify_hypertable_paths(root, output_rel->pathlist, input_rel);
-
-		if (parse->hasAggs && stage == UPPERREL_GROUP_AGG)
-		{
-			/* Existing AggPaths are modified here.
-			 * No new AggPaths should be added after this if there
-			 * are partials. */
-			partials_found = ts_plan_process_partialize_agg(root, output_rel);
-		}
 	}
 
 	if (stage == UPPERREL_GROUP_AGG && output_rel != NULL && ts_guc_enable_optimizations &&
@@ -1730,9 +1702,6 @@ timescaledb_create_upper_paths_hook(PlannerInfo *root, UpperRelationKind stage,
 	{
 		if (parse->hasAggs)
 			ts_preprocess_first_last_aggregates(root, root->processed_tlist);
-
-		if (!partials_found)
-			ts_plan_add_hashagg(root, input_rel, output_rel);
 	}
 
 	if (ts_cm_functions->create_upper_paths_hook != NULL)
