@@ -251,16 +251,29 @@ ts_sort_transform_expr(Expr *orig_expr)
  */
 
 static EquivalenceClass *
-sort_transform_ec(PlannerInfo *root, EquivalenceClass *orig)
+sort_transform_ec(PlannerInfo *root, EquivalenceClass *orig, Relids child_relids)
 {
-	ListCell *lc_member;
 	EquivalenceClass *newec = NULL;
 	bool propagate_to_children = false;
 
 	/* check all members, adding only transformable members to new ec */
+	EquivalenceMember *ec_mem;
+#if PG18_GE
+	/* Use specialized iterator to include child ems.
+	 *
+	 * https://github.com/postgres/postgres/commit/d69d45a5
+	 */
+	EquivalenceMemberIterator it;
+
+	setup_eclass_member_iterator(&it, orig, child_relids);
+	while ((ec_mem = eclass_member_iterator_next(&it)) != NULL)
+	{
+#else
+	ListCell *lc_member;
 	foreach (lc_member, orig->ec_members)
 	{
-		EquivalenceMember *ec_mem = (EquivalenceMember *) lfirst(lc_member);
+		ec_mem = (EquivalenceMember *) lfirst(lc_member);
+#endif
 		Expr *transformed_expr = ts_sort_transform_expr(ec_mem->em_expr);
 
 		if (transformed_expr != ec_mem->em_expr)
@@ -317,8 +330,12 @@ sort_transform_ec(PlannerInfo *root, EquivalenceClass *orig)
 				newec->ec_opfamilies = opfamilies;
 				newec->ec_collation = orig->ec_collation;
 				newec->ec_members = NIL;
+#if PG18_GE
+				newec->ec_childmembers = NULL;
+				newec->ec_childmembers_size = 0;
+#endif
 				newec->ec_sources = list_copy(orig->ec_sources);
-				newec->ec_derives = list_copy(orig->ec_derives);
+				newec->ec_derives_list = list_copy(orig->ec_derives_list);
 				newec->ec_relids = bms_copy(orig->ec_relids);
 				newec->ec_has_const = orig->ec_has_const;
 
@@ -347,7 +364,26 @@ sort_transform_ec(PlannerInfo *root, EquivalenceClass *orig)
 				 */
 				orig->ec_has_volatile = false;
 			}
+#if PG18_LT
 			newec->ec_members = lappend(newec->ec_members, em);
+#else
+			/* Update the child member lists when adding child ems.
+			 *
+			 * https://github.com/postgres/postgres/commit/d69d45a5
+			 */
+			if (em->em_is_child)
+				ts_add_child_eq_member(root, newec, em, it.current_relid);
+			else
+				newec->ec_members = lappend(newec->ec_members, em);
+
+#endif
+			int i = -1;
+			while ((i = bms_next_member(em->em_relids, i)) >= 0)
+			{
+				RelOptInfo *rel = root->simple_rel_array[i];
+
+				rel->eclass_indexes = bms_add_member(rel->eclass_indexes, root->eq_classes->length);
+			}
 		}
 	}
 	/* if any transforms were found return new ec */
@@ -438,15 +474,34 @@ ts_sort_transform_get_pathkeys(PlannerInfo *root, RelOptInfo *rel, RangeTblEntry
 		desired_ec_relid = appinfo->parent_relid;
 	}
 
-	if (!bms_is_member(desired_ec_relid, last_pk->pk_eclass->ec_relids))
+	EquivalenceClass *last_pk_eclass = last_pk->pk_eclass;
+
+	if (!bms_is_member(desired_ec_relid, last_pk_eclass->ec_relids))
 	{
 		return NIL;
 	}
 
+	Relids child_relids = NULL;
+#if PG18_GE
+	/* In PG18, iterating over child ems requires you to
+	 * use child relids with a special iterator. Here we gather
+	 * them by collecting them from childmembers array.
+	 *
+	 * https://github.com/postgres/postgres/commit/d69d45a5
+	 */
+	for (int i = 0; i < last_pk_eclass->ec_childmembers_size; i++)
+	{
+		if (list_length(last_pk_eclass->ec_childmembers[i]) > 0)
+		{
+			child_relids = bms_add_member(child_relids, i);
+		}
+	}
+#endif
+
 	/*
 	 * Try to apply the transformation.
 	 */
-	transformed = sort_transform_ec(root, last_pk->pk_eclass);
+	transformed = sort_transform_ec(root, last_pk_eclass, child_relids);
 
 	if (transformed == NULL)
 		return NIL;
@@ -454,7 +509,7 @@ ts_sort_transform_get_pathkeys(PlannerInfo *root, RelOptInfo *rel, RangeTblEntry
 	new_pk = make_canonical_pathkey(root,
 									transformed,
 									last_pk->pk_opfamily,
-									last_pk->pk_strategy,
+									last_pk->pk_cmptype,
 									last_pk->pk_nulls_first);
 
 	/*
@@ -502,13 +557,18 @@ ts_sort_transform_replace_pathkeys(void *node, List *transformed_pathkeys, List
 
 	if (IsA(path, CustomPath))
 	{
-		/*
-		 * We should only see ChunkAppend here.
-		 */
 		CustomPath *custom = castNode(CustomPath, path);
 		ts_sort_transform_replace_pathkeys(custom->custom_paths,
 										   transformed_pathkeys,
 										   original_pathkeys);
+
+		/* Need to handle tsl-specific custom path types */
+		if (ts_cm_functions->sort_transform_replace_pathkeys != NULL)
+		{
+			ts_cm_functions->sort_transform_replace_pathkeys(path,
+															 transformed_pathkeys,
+															 original_pathkeys);
+		}
 	}
 	else if (IsA(path, MergeAppendPath))
 	{
