@@ -460,7 +460,7 @@ add_job_stats_by_job_type(JsonbParseState *parse_state)
 		"	MAX(consecutive_crashes) AS max_consecutive_crashes "
 		"FROM "
 		"	_timescaledb_internal.bgw_job_stat s "
-		"	JOIN _timescaledb_config.bgw_job j on j.id = s.job_id "
+		"	JOIN _timescaledb_catalog.bgw_job j on j.id = s.job_id "
 		"GROUP BY job_type "
 		"ORDER BY job_type";
 
@@ -980,11 +980,13 @@ build_telemetry_report()
 	pushJsonbValue(&parse_state, WJB_END_OBJECT, NULL);
 
 	/* add tuned info, which is optional */
-	if (ts_last_tune_time != NULL)
-		ts_jsonb_add_str(parse_state, REQ_TS_LAST_TUNE_TIME, ts_last_tune_time);
+	char *last_tune_time = GetConfigOptionByName("timescaledb.last_tune_time", NULL, true);
+	if (last_tune_time != NULL)
+		ts_jsonb_add_str(parse_state, REQ_TS_LAST_TUNE_TIME, last_tune_time);
 
-	if (ts_last_tune_version != NULL)
-		ts_jsonb_add_str(parse_state, REQ_TS_LAST_TUNE_VERSION, ts_last_tune_version);
+	char *last_tune_version = GetConfigOptionByName("timescaledb.last_tune_version", NULL, true);
+	if (last_tune_version != NULL)
+		ts_jsonb_add_str(parse_state, REQ_TS_LAST_TUNE_VERSION, last_tune_version);
 
 	/* add cloud to telemetry when set */
 	if (ts_telemetry_cloud != NULL)
