@@ -9,7 +9,7 @@ CREATE TABLE target (
    time        TIMESTAMPTZ       NOT NULL,
    location    SMALLINT          NOT NULL,
    temperature DOUBLE PRECISION  NULL,
-   val text default 'string -'
+   to_be_dropped text
 );
 
 SELECT create_hypertable(
@@ -26,6 +26,14 @@ FROM generate_series(
   ) as time,
 generate_series(1,4) as location;
 
+-- This makes sure we have one column with attisdropped and one column
+-- with atthasmissing set to true. These two cases can cause problems
+-- with chunk dispatch execution when merging using a when-clause with
+-- inserts. Unfortunately they are hard to trigger, so this is not a
+-- definitive test.
+ALTER TABLE target DROP COLUMN to_be_dropped;
+ALTER TABLE target ADD COLUMN val text default 'string -';
+
 -- Create source table with location and temperature
 CREATE TABLE source (
    time        TIMESTAMPTZ       NOT NULL,
